<?php
/* --------------------------------------------------------------
   ParcelServiceReader.inc.php 2018-07-06
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class ParcelServiceReader
 */
class GXParcelServiceReader implements ParcelServiceReaderInterface
{
	/**
	 * @var \CI_DB_query_builder
	 */
	protected $db;
	
	
	/**
	 * GXParcelServiceReader constructor.
	 *
	 * @param \CI_DB_query_builder $db
	 */
	public function __construct(CI_DB_query_builder $db)
	{
		$this->db = $db;
	}
	
	
	/**
	 * Returns all parcel services limited by given limit and offset.
	 *
	 * @param \IntType $limit
	 * @param \IntType $offset
	 *
	 * @return array
	 */
	public function fetchAll(\IntType $limit, \IntType $offset)
	{
		$parcelServicesData = $this->db->select()
		                               ->from('parcel_services')
		                               ->limit($limit->asInt(), $offset->asInt())
		                               ->get()
		                               ->result_array();
		
		return $this->_prepareParcelServicesData($parcelServicesData);
	}
	
	
	/**
	 * Returns the parcel service search result limited by given limit and offset as an array.
	 *
	 * @param \IntType    $limit
	 * @param \IntType    $offset
	 * @param \StringType $searchCondition
	 *
	 * @return array
	 */
	public function fetchBy(\IntType $limit, \IntType $offset, \StringType $searchCondition)
	{
		$parcelServicesData = $this->db->select()
		                               ->from('parcel_services')
		                               ->limit($limit->asInt(), $offset->asInt())
		                               ->where($searchCondition->asString())
		                               ->get()
		                               ->result_array();
		
		return $this->_prepareParcelServicesData($parcelServicesData);
	}
	
	
	/**
	 * Returns a parcel service as an array identified by its ID.
	 *
	 * @param \ParcelServiceId $parcelServiceId
	 *
	 * @return array
	 */
	public function fetchById(\ParcelServiceId $parcelServiceId)
	{
		$rawData = $this->db->select()
		                    ->from('parcel_services')
		                    ->join('parcel_services_description',
		                           'parcel_services.parcel_service_id = parcel_services_description.parcel_service_id')
		                    ->where('parcel_services.parcel_service_id', $parcelServiceId->id())
		                    ->get()
		                    ->result_array();
		
		return $this->_prepareParcelData($rawData);
	}
	
	
	protected function _prepareParcelServicesData(array $parcelServicesData)
	{
		foreach($parcelServicesData as &$parcelServiceData)
		{
			$parcelServiceData['descriptions'] = $this->db->select()
			                                              ->from('parcel_services_description')
			                                              ->where('parcel_service_id',
			                                                      $parcelServiceData['parcel_service_id'])
			                                              ->get()
			                                              ->result_array();
		}
		
		return $parcelServicesData;
	}
	
	
	protected function _prepareParcelData(array $parcelData)
	{
		$parcelServiceData = [];
		$descriptions      = [];
		
		foreach($parcelData as $parcelDataSet)
		{
			$parcelServiceData = [
				'parcel_service_id' => $parcelDataSet['parcel_service_id'],
				'name'              => $parcelDataSet['name'],
				'default'           => $parcelDataSet['default']
			];
			
			$descriptions[] = [
				'parcel_service_id' => $parcelDataSet['parcel_service_id'],
				'language_id'       => $parcelDataSet['language_id'],
				'url'               => $parcelDataSet['url'],
				'comment'           => $parcelDataSet['comment']
			];
		}
		
		if(!count($parcelServiceData))
		{
			return null;
		}
		
		return array_merge($parcelServiceData, [
			'descriptions' => $descriptions
		]);
	}
}